@@ -18,7 +18,9 @@ class HeaderParser:
 
     def parse(self):
         header_data = {}
-        header_data["from"] = self._get_addr("from")
+        from_name, from_addr = self._get_from()
+        header_data["from_name"] = from_name
+        header_data["from"] = " ".join([from_name, from_addr])
         header_data["to"] = self._get_addr("to")
         header_data["subject"] = self._get_subject()
         header_data["date"] = self._get_date()
@@ -32,6 +34,26 @@ class HeaderParser:
             header_data["bcc"] = bcc
 
         return header_data
+    
+    def _get_from(self):
+        try:
+            addr_list = email.utils.getaddresses(self.msg.get_all("from", []))
+            assert len(addr_list) <= 1
+
+            if(addr_list):
+                name, address = addr_list[0]
+                name_dec, charset = email.header.decode_header(name)[0]
+                name_str = self._decode(name_dec, charset, "from")
+
+                address_dec, charset = email.header.decode_header(address)[0]
+                address_str = self._decode(address_dec, charset, "from")
+
+                return name_str, address_str
+            return "", ""
+        except:
+            exc_type, value, _ = sys.exc_info()
+            self.fails["from"] = "{}: {}".format(exc_type.__name__, value)
+            return "", ""
 
     def _get_addr(self, field):
         try:
@@ -42,14 +64,7 @@ class HeaderParser:
                 addr_items = []
                 for item in addr_tuple:
                     item_dec, charset = email.header.decode_header(item)[0]
-
-                    if charset:
-                        item_str = item_dec.decode(charset)
-                    elif type(item_dec) == bytes:
-                        item_str = item_dec.decode()
-                    else:
-                        assert type(item) == str
-                        item_str = item_dec
+                    item_str = self._decode(item_dec, charset, "subject")
 
                     if(item_str):
                         addr_items.append(item_str)
@@ -69,15 +84,7 @@ class HeaderParser:
 
                 subject_items = []
                 for item_dec, charset in subject_parts:
-                    if charset:
-                        item_str = item_dec.decode(charset)
-                    # charset is None for non-encoded header parts and the part itself a bytestring
-                    elif type(item_dec) == bytes:
-                        item_str = item_dec.decode()
-                    else:
-                        # if whole header is non-encoded, result is a string
-                        assert type(item_dec) == str
-                        item_str = item_dec
+                    item_str = self._decode(item_dec, charset, "subject")
 
                     subject_items.append(item_str.strip())
 
@@ -106,6 +113,26 @@ class HeaderParser:
                 if(decoded_date):
                     return email.utils.parsedate_to_datetime(decoded_date)
 
+    def _decode(self, item, charset, field):
+        try:
+            item_str = ""
+            if charset:
+                item_str = item.decode(charset)
+                # charset is None for non-encoded header parts and the part itself a bytestring
+            elif type(item) == bytes:
+                item_str = item.decode()
+            else:
+                # if whole header is non-encoded, result is a string
+                assert type(item) == str
+                item_str = item
+
+            return item_str
+        except:
+                exc_type, value, _ = sys.exc_info()
+                self.fails[field] = "{}: {}".format(exc_type.__name__, value)
+                return ""
+
+
 class BodyParser:
     def __init__(self, msg):
         self.msg = msg
